@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+ * Copyright (c) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2008
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -174,16 +174,11 @@ C_family(const char *file, int type)
 					}
 				}
 			} else {
-				if (dflag) {
-					if (target == REF) {
-						if (defined(token))
-							PUT(token, lineno, sp);
-					} else if (target == SYM) {
-						if (!defined(token))
-							PUT(token, lineno, sp);
-					}
-				} else {
-					if (target == SYM)
+				if (target == REF) {
+					if (defined(token))
+						PUT(token, lineno, sp);
+				} else if (target == SYM) {
+					if (!defined(token))
 						PUT(token, lineno, sp);
 				}
 			}
@@ -282,13 +277,8 @@ C_family(const char *file, int type)
 				if (c == '\n')
 					pushbacktoken();
 			} else {
-				if (dflag) {
-					if (target == DEF)
-						PUT(token, lineno, sp);
-				} else {
-					if (target == SYM)
-						PUT(token, lineno, sp);
-				}
+				if (target == DEF)
+					PUT(token, lineno, sp);
 			}
 			break;
 		case SHARP_IMPORT:
@@ -315,14 +305,51 @@ C_family(const char *file, int type)
 			(void)nexttoken(interested, c_reserved_word);
 			break;
 		case C_STRUCT:
+		case C_ENUM:
+		case C_UNION:
 			c = nexttoken(interested, c_reserved_word);
-			if (c == '{' /* } */) {
+			if (c == SYMBOL) {
+				if (peekc(0) == '{') /* } */ {
+					if (target == DEF)
+						PUT(token, lineno, sp);
+				} else if (target == REF) {
+					if (defined(token))
+						PUT(token, lineno, sp);
+				} else if (target == SYM) {
+					if (!defined(token))
+						PUT(token, lineno, sp);
+				}
+				c = nexttoken(interested, c_reserved_word);
+			}
+			if (c == '{' /* } */ && cc == C_ENUM) {
+				int savelevel = level;
+
+				for (; c != EOF; c = nexttoken(interested, c_reserved_word)) {
+					switch (c) {
+					case SHARP_IFDEF:
+					case SHARP_IFNDEF:
+					case SHARP_IF:
+					case SHARP_ELIF:
+					case SHARP_ELSE:
+					case SHARP_ENDIF:
+						condition_macro(c, target);
+						continue;
+					default:
+						break;
+					}
+					if (c == '{')
+						level++;
+					else if (c == '}') {
+						if (--level == savelevel)
+							break;
+					} else if (c == SYMBOL) {
+						if (target == DEF)
+							PUT(token, lineno, sp);
+					}
+				}
+			} else {
 				pushbacktoken();
-				break;
 			}
-			if (c == SYMBOL)
-				if (target == SYM)
-					PUT(token, lineno, sp);
 			break;
 		/* control statement check */
 		case C_BREAK:
@@ -341,7 +368,11 @@ C_family(const char *file, int type)
 				warning("Out of function. %8s [+%d %s]", token, lineno, curfile);
 			break;
 		case C_TYPEDEF:
-			if (tflag) {
+			{
+				/*
+				 * This parser is too complex to maintain.
+				 * We should rewrite the whole.
+				 */
 				char savetok[MAXTOKEN];
 				int savelineno = 0;
 				int typedef_savelevel = level;
@@ -363,8 +394,16 @@ C_family(const char *file, int type)
 					c = nexttoken(interest_enum, c_reserved_word);
 					/* read enum name if exist */
 					if (c == SYMBOL) {
-						if (target == SYM)
-							PUT(token, lineno, sp);
+						if (peekc(0) == '{') /* } */ {
+							if (target == DEF)
+								PUT(token, lineno, sp);
+						} else if (target == REF) {
+							if (defined(token))
+								PUT(token, lineno, sp);
+						} else if (target == SYM) {
+							if (!defined(token))
+								PUT(token, lineno, sp);
+						}
 						c = nexttoken(interest_enum, c_reserved_word);
 					}
 					for (; c != EOF; c = nexttoken(interest_enum, c_reserved_word)) {
@@ -393,7 +432,7 @@ C_family(const char *file, int type)
 							if (c_ == C_ENUM) {
 								if (target == DEF && level > typedef_savelevel)
 									PUT(token, lineno, sp);
-								if (target == SYM && level == typedef_savelevel)
+								if (target == SYM && level == typedef_savelevel && !defined(token))
 									PUT(token, lineno, sp);
 							} else {
 								if (target == REF) {
@@ -677,10 +716,10 @@ condition_macro(int cc, int target)
 	while ((cc = nexttoken(NULL, c_reserved_word)) != EOF && cc != '\n') {
                 if (cc == SYMBOL && strcmp(token, "defined") != 0) {
 			if (target == REF) {
-				if (dflag && defined(token))
+				if (defined(token))
 		                        PUT(token, lineno, sp);
 			} else if (target == SYM) {
-				if (!dflag || !defined(token))
+				if (!defined(token))
 		                	PUT(token, lineno, sp);
 			}
 		}
