@@ -1,22 +1,21 @@
 /*
  * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
- *	Tama Communications Corporation
+ *	2008	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -30,6 +29,9 @@
 #else
 #include <strings.h>
 #endif
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
 #include "global.h"
 #include "anchor.h"
 #include "htags.h"
@@ -142,25 +144,31 @@ anchor_load(const char *path)
 				break;
 			}
 			if (db == GTAGS) {
-				char *p;
+				char *p = ptable.part[PART_LINE].start;
 
-				for (p = ptable.part[PART_LINE].start; *p && isspace((unsigned char)*p); p++)
+				for (; *p && isspace((unsigned char)*p); p++)
 					;
 				if (!*p) {
 					recover(&ptable);
 					die("The output of parser is illegal.\n%s", ctags_x);
 				}
-				type = 'D';
-				if (*p == '#') {
-					for (p++; *p && isspace((unsigned char)*p); p++)
-						;
-					if (*p) {
-						if (locatestring(p, "define", MATCH_AT_FIRST) ||
-						    locatestring(p, "undef", MATCH_AT_FIRST))
-							type = 'M';
-					}
-				} else if (locatestring(p, "typedef", MATCH_AT_FIRST))
+				/*
+				 * Function header is applied only to the anchor whoes type is 'D'.
+				 * (D: function, M: macro, T: type)
+				 */
+				type = 'T';
+				if (*p == '#')
+					type = 'M';
+				else if (locatestring(p, "typedef", MATCH_AT_FIRST))
 					type = 'T';
+				else if ((p = locatestring(p, ptable.part[PART_TAG].start, MATCH_FIRST)) != NULL) {
+					/* skip a tag and the following blanks */
+					p += strlen(ptable.part[PART_TAG].start);
+					for (; *p && isspace((unsigned char)*p); p++)
+						;
+					if (*p == '(')
+						type = 'D';
+				}
 			}  else if (db == GRTAGS)
 				type = 'R';
 			else
