@@ -3,19 +3,18 @@
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -134,6 +133,11 @@ repeat_find_next(void)
 /*
  * Common processing for each XARGS_XXXX type.
  */
+#ifdef _WIN32
+#define QUOTE	'"'
+#else
+#define QUOTE	'\''
+#endif
 #define APPEND_ARGUMENT(p) {\
 	char *path = (p);\
 	length = strlen(path);\
@@ -142,7 +146,7 @@ repeat_find_next(void)
 			gpath_put(path, GPATH_OTHER);\
 		continue;\
 	}\
-	if (strbuf_getlen(comline) + length > limit)\
+	if (strbuf_getlen(comline) + length + 2 > limit)\
 		break;\
 	xp->seqno++;\
 	if (xp->put_gpath)\
@@ -154,7 +158,9 @@ repeat_find_next(void)
 		if (xp->verbose)\
 			xp->verbose(path + 2, xp->seqno, 0);\
 		strbuf_putc(comline, ' ');\
+		strbuf_putc(comline, QUOTE);\
 		strbuf_puts(comline, path);\
+		strbuf_putc(comline, QUOTE);\
 		count++;\
 	}\
 }
